@@ -1,13 +1,11 @@
 BUILD_BIT='64bit'
 BUILD_CPU='unknown'
-BUILD_DATE='2019-03-31'
+BUILD_DATE='2019-05-09'
 BUILD_MACHINE='aarch64'
 BUILD_OS='Linux   '
-BUILD_TIME='21:42'
-BUILT_BY='builduser'
-BUILT_ON='leming'
+BUILD_TIME='22:17'
 COMPILER_VERSION='aarch64-unknown-linux-gnu-gcc (GCC) 8.2.1 20180831'
-CYTHON_VERSION='0.29.6'
+CYTHON_VERSION='0.29.7'
 LINKER_VERSION='GNU ld (GNU Binutils) 2.31.1'
 PYTHON_VERSION='2.7.16'
 RELEASE_BUILD=True
