@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -12,6 +12,7 @@ from xpra.client.client_widget_base import ClientWidgetBase
 from xpra.os_util import bytestostr, PYTHON2, PYTHON3, OSX, WIN32
 from xpra.util import typedict, envbool, WORKSPACE_UNSET, WORKSPACE_NAMES
 from xpra.log import Logger
+
 log = Logger("window")
 plog = Logger("paint")
 focuslog = Logger("focus")
@@ -26,16 +27,28 @@ iconlog = Logger("icon")
 REPAINT_ALL = os.environ.get("XPRA_REPAINT_ALL", "")
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
 PROPERTIES_DEBUG = [x.strip() for x in os.environ.get("XPRA_WINDOW_PROPERTIES_DEBUG", "").split(",")]
+AWT_DIALOG_WORKAROUND = envbool("XPRA_AWT_DIALOG_WORKAROUND", WIN32)
+SET_SIZE_CONSTRAINTS = envbool("XPRA_SET_SIZE_CONSTRAINTS", True)
 
 
 class ClientWindowBase(ClientWidgetBase):
 
-    def __init__(self, client, group_leader, watcher_pid, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, border, max_window_size, default_cursor_data, pixel_depth):
-        log("%s%s", type(self), (client, group_leader, watcher_pid, wid, x, y, ww, wh, bw, bh, metadata, override_redirect, client_properties, max_window_size, default_cursor_data, pixel_depth))
+    #(overriden in subclasses)
+    NAME_TO_HINT = {}
+
+    def __init__(self, client, group_leader, watcher_pid, wid,
+                 wx, wy, ww, wh, bw, bh,
+                 metadata, override_redirect, client_properties,
+                 border, max_window_size, default_cursor_data, pixel_depth):
+        log("%s%s", type(self),
+            (client, group_leader, watcher_pid, wid,
+             wx, wy, ww, wh, bw, bh,
+             metadata, override_redirect, client_properties,
+             border, max_window_size, default_cursor_data, pixel_depth))
         ClientWidgetBase.__init__(self, client, watcher_pid, wid, metadata.boolget("has-alpha"))
         self._override_redirect = override_redirect
         self.group_leader = group_leader
-        self._pos = (x, y)
+        self._pos = (wx, wy)
         self._size = (ww, wh)
         self._client_properties = client_properties
         self._set_initial_position = metadata.boolget("set-initial-position", False)
@@ -83,7 +96,8 @@ class ClientWindowBase(ClientWidgetBase):
         self._desktop_workspace = self.get_desktop_workspace()
         def wn(w):
             return WORKSPACE_NAMES.get(w, w)
-        workspacelog("init_window(..) workspace=%s, current workspace=%s", wn(self._window_workspace), wn(self._desktop_workspace))
+        workspacelog("init_window(..) workspace=%s, current workspace=%s",
+                     wn(self._window_workspace), wn(self._desktop_workspace))
         if self.max_window_size and b"size-constraints" not in metadata:
             #this ensures that we will set size-constraints and honour max_window_size:
             metadata[b"size-constraints"] = {}
@@ -130,7 +144,7 @@ class ClientWindowBase(ClientWidgetBase):
                 #ie: "encodings.rgb_formats" -> "rgb_formats"
                 #ie: "encoding.full_csc_modes" -> "full_csc_modes"
                 ek = k.split(".", 1)[1]
-            except:
+            except IndexError:
                 ek = k
             dv = encoding_defaults.get(ek)
             if dv is not None and dv==v:
@@ -187,7 +201,7 @@ class ClientWindowBase(ClientWidgetBase):
         #WARNING: "class-instance" needs to go first because others may realize the window
         #(and GTK doesn't set the "class-instance" once the window is realized)
         if b"class-instance" in metadata:
-            self.set_class_instance(*self._metadata.strlistget("class-instance", ("xpra", "Xpra")))
+            self.set_class_instance(*self._metadata.strlistget("class-instance", ("xpra", "Xpra"), 2, 2))
             self.reset_icon()
 
         if b"title" in metadata:
@@ -209,7 +223,7 @@ class ClientWindowBase(ClientWidgetBase):
                         if PYTHON2:
                             value = value.decode("utf-8")
                         return value
-                    title = re.sub("@[\w\-]*@", metadata_replace, title)
+                    title = re.sub(r"@[\w\-]*@", metadata_replace, title)
                 if PYTHON2:
                     utf8_title = title.encode("utf-8")
                 else:
@@ -228,9 +242,10 @@ class ClientWindowBase(ClientWidgetBase):
             self.reset_icon()
 
         if b"size-constraints" in metadata:
-            self.size_constraints = typedict(metadata.dictget("size-constraints"))
-            self._set_initial_position = self.size_constraints.boolget("set-initial-position", self._set_initial_position)
-            self.set_size_constraints(self.size_constraints, self.max_window_size)
+            sc = typedict(metadata.dictget("size-constraints"))
+            self.size_constraints = sc
+            self._set_initial_position = sc.boolget("set-initial-position", self._set_initial_position)
+            self.set_size_constraints(sc, self.max_window_size)
 
         if b"set-initial-position" in metadata:
             #this should be redundant - but we keep it here for consistency
@@ -344,7 +359,7 @@ class ClientWindowBase(ClientWidgetBase):
             skip_pager = metadata.boolget("skip-pager")
             if self._skip_pager!=skip_pager:
                 self._skip_pager = skip_pager
-                self.set_skip_taskbar_hint(skip_pager)
+                self.set_skip_pager_hint(skip_pager)
 
         if b"workspace" in metadata:
             self.set_workspace(metadata.intget("workspace"))
@@ -383,7 +398,7 @@ class ClientWindowBase(ClientWidgetBase):
     def set_bypass_compositor(self, v):
         pass        #see gtk client window base
 
-    def set_strut(self, d):
+    def set_strut(self, strut):
         pass        #see gtk client window base
 
     def set_fullscreen_monitors(self, fsm):
@@ -394,26 +409,38 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def set_size_constraints(self, size_constraints, max_window_size):
+        if not SET_SIZE_CONSTRAINTS:
+            return
         geomlog("set_size_constraints(%s, %s)", size_constraints, max_window_size)
         hints = typedict()
+        client = self._client
         for (a, h1, h2) in [
             (b"maximum-size", b"max_width", b"max_height"),
             (b"minimum-size", b"min_width", b"min_height"),
             (b"base-size", b"base_width", b"base_height"),
             (b"increment", b"width_inc", b"height_inc"),
             ]:
-            if a in (b"base-size", b"increment"):
-                def closetoint(v):
-                    return abs(int(v)-v)<0.00001
-                int_scaling = closetoint(self._client.xscale) and closetoint(self._client.yscale)
-                if not int_scaling:
-                    #don't scale increment and base size constraints with non integer scaling values
-                    #as this would use rounding
-                    continue
             v = size_constraints.intpair(a)
+            geomlog("intpair(%s)=%s", a, v)
             if v:
                 v1, v2 = v
-                hints[h1], hints[h2] = self._client.sp(v1, v2)
+                sv1 = client.sx(v1)
+                sv2 = client.sy(v2)
+                if a in (b"base-size", b"increment"):
+                    #rounding is not allowed for these values
+                    fsv1 = client.fsx(v1)
+                    fsv2 = client.fsy(v2)
+                    def closetoint(v):
+                        #tolerate some rounding error:
+                        #(ie: 2:3 scaling may not give an integer without a tiny bit of rounding)
+                        return abs(int(v)-v)<0.00001
+                    if not closetoint(fsv1) or not closetoint(fsv2):
+                        #the scaled value is not close to an int,
+                        #so we can't honour it:
+                        geomlog("cannot honour '%s' due to scaling, scaled values are not integers: %s, %s",
+                                a, fsv1, fsv2)
+                        continue
+                hints[h1], hints[h2] = sv1, sv2
         if not OSX:
             for (a, h) in [
                 (b"minimum-aspect-ratio", b"min_aspect"),
@@ -448,17 +475,21 @@ class ClientWindowBase(ClientWidgetBase):
             #not honouring "base" + "inc", but honouring just "min" instead:
             maxw = max(minw, maxw)
             maxh = max(minh, maxh)
-            geomlog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s", max_window_size, hints, rw, rh, maxw, maxh)
+            geomlog("modified hints for max window size %s: %s (rw=%s, rh=%s) -> max=%sx%s",
+                    max_window_size, hints, rw, rh, maxw, maxh)
             #ensure we don't have duplicates with bytes / strings,
             #and that keys are always "bytes":
             #(in practice this code should never fire, just here as a reminder)
             for x in ("max_width", "max_height"):
                 try:
                     del hints[x]
-                except:
+                except KeyError:
                     pass
-            hints[b"max_width"] = maxw
-            hints[b"max_height"] = maxh
+            #bug 2214: GTK3 on win32 gets confused if we specify a large max-size
+            # and it will mess up maximizing the window
+            if not WIN32 or PYTHON2 or (maxw<32000 or maxh<32000):
+                hints[b"max_width"] = maxw
+                hints[b"max_height"] = maxh
         try:
             geomlog("calling: %s(%s)", self.apply_geometry_hints, hints)
             #save them so the window hooks can use the last value used:
@@ -478,6 +509,14 @@ class ClientWindowBase(ClientWidgetBase):
     def set_window_type(self, window_types):
         hints = 0
         for window_type in window_types:
+            #win32 workaround:
+            if AWT_DIALOG_WORKAROUND and window_type=="DIALOG" and self._metadata.boolget("skip-taskbar"):
+                wm_class = self._metadata.strlistget("class-instance", (None, None), 2, 2)
+                if wm_class and len(wm_class)==2 and wm_class[0] and wm_class[0].startswith("sun-awt-X11"):
+                    #replace "DIALOG" with "NORMAL":
+                    if "NORMAL" in window_types:
+                        continue
+                    window_type = "NORMAL"
             hint = self.NAME_TO_HINT.get(window_type, None)
             if hint is not None:
                 hints |= hint
@@ -579,7 +618,7 @@ class ClientWindowBase(ClientWidgetBase):
         #the "ClientWindow"
         self._client.send_refresh_all()
 
-    def draw_region(self, x, y, width, height, coding, img_data, rowstride, packet_sequence, options, callbacks):
+    def draw_region(self, x, y, width, height, coding, img_data, rowstride, _packet_sequence, options, callbacks):
         """ Note: this runs from the draw thread (not UI thread) """
         backing = self._backing
         if not backing:
@@ -588,7 +627,8 @@ class ClientWindowBase(ClientWidgetBase):
             fire_paint_callbacks(callbacks, -1, "no backing")
             return
         def after_draw_refresh(success, message=""):
-            plog("after_draw_refresh(%s, %s) %sx%s at %sx%s encoding=%s, options=%s", success, message, width, height, x, y, coding, options)
+            plog("after_draw_refresh(%s, %s) %sx%s at %sx%s encoding=%s, options=%s",
+                 success, message, width, height, x, y, coding, options)
             if success<=0:
                 return
             backing = self._backing
@@ -607,14 +647,20 @@ class ClientWindowBase(ClientWidgetBase):
             callbacks.append(after_draw_refresh)
         backing.draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
 
-    def spinner(self, ok):
+    def eos(self):
+        """ Note: this runs from the draw thread (not UI thread) """
+        backing = self._backing
+        if backing:
+            backing.eos()
+
+    def spinner(self, _ok):
         if not self.can_have_spinner():
             return
         log("spinner(%s) queueing redraw")
         #with normal windows, we just queue a draw request
         #and let the expose event paint the spinner
         w, h = self.get_size()
-        self.queue_draw(self, 0, 0, w, h)
+        self.queue_draw(0, 0, w, h)
 
     def can_have_spinner(self):
         if self._backing is None:
@@ -671,7 +717,7 @@ class ClientWindowBase(ClientWidgetBase):
             log.error(" this server does not support dbus-proxying")
             return
         rpc_args = [self._id]+args
-        return self._client.rpc_call("dbus", rpc_args, **kwargs)
+        self._client.rpc_call("dbus", rpc_args, **kwargs)
 
 
     def get_mouse_event_wid(self, _x, _y):
@@ -679,26 +725,30 @@ class ClientWindowBase(ClientWidgetBase):
         return self._id
 
     def do_motion_notify_event(self, event):
-        if self._client.readonly or self._client.server_readonly:
+        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
             return
-        pointer, modifiers, buttons = self._pointer_modifiers(event)
+        pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
         wid = self.get_mouse_event_wid(*pointer)
-        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", event, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
-        self._client.send_mouse_position(["pointer-position", wid,
-                                          pointer, modifiers, buttons])
+        mouselog("do_motion_notify_event(%s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, relative pointer=%s, modifiers=%s, buttons=%s", event, wid, self._client._focused, self._id, self._device_info(event), pointer, relative_pointer, modifiers, buttons)
+        pdata = pointer
+        if self._client.server_pointer_relative:
+            pdata = list(pointer)+list(relative_pointer)
+        packet = ["pointer-position", wid, pdata, modifiers, buttons]
+        self._client.send_mouse_position(packet)
 
     def _device_info(self, event):
         try:
             return event.device.get_name()
-        except:
+        except AttributeError:
             return ""
 
     def _button_action(self, button, event, depressed, *args):
-        if self._client.readonly or self._client.server_readonly:
+        if self._client.readonly or self._client.server_readonly or not self._client.server_pointer:
             return
-        pointer, modifiers, buttons = self._pointer_modifiers(event)
+        pointer, relative_pointer, modifiers, buttons = self._pointer_modifiers(event)
         wid = self.get_mouse_event_wid(*pointer)
-        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s", button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
+        mouselog("_button_action(%s, %s, %s) wid=%s / focus=%s / window wid=%i, device=%s, pointer=%s, modifiers=%s, buttons=%s",
+                 button, event, depressed, wid, self._client._focused, self._id, self._device_info(event), pointer, modifiers, buttons)
         #map wheel buttons via translation table to support inverted axes:
         server_button = button
         if button>3:
@@ -713,8 +763,11 @@ class ClientWindowBase(ClientWidgetBase):
                     continue
                 b = sb
             server_buttons.append(b)
+        pdata = pointer
+        if self._client.server_pointer_relative:
+            pdata = list(pointer)+list(relative_pointer)
         def send_button(pressed):
-            self._client.send_button(wid, server_button, pressed, pointer, modifiers, server_buttons, *args)
+            self._client.send_button(wid, server_button, pressed, pdata, modifiers, server_buttons, *args)
         pressed_state = self.button_state.get(button, False)
         if SIMULATE_MOUSE_DOWN and pressed_state is False and depressed is False:
             mouselog("button action: simulating a missing mouse-down event for window %s before sending the mouse-up event", wid)
